<?php

namespace Event\Event\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Event\Event\Http\Requests\PrizeRequest;
use Event\Event\Interfaces\PrizeRepositoryInterface;
use Event\Event\Models\Prize;

/**
 * Resource controller class for prize.
 */
class PrizeResourceController extends BaseController
{

    /**
     * Initialize prize resource controller.
     *
     * @param type PrizeRepositoryInterface $prize
     *
     * @return null
     */
    public function __construct(PrizeRepositoryInterface $prize)
    {
        parent::__construct();
        $this->repository = $prize;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Event\Event\Repositories\Criteria\PrizeResourceCriteria::class);
    }

    /**
     * Display a list of prize.
     *
     * @return Response
     */
    public function index(PrizeRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Event\Event\Repositories\Presenter\PrizePresenter::class)
                ->$function();
        }

        $prizes = $this->repository->paginate();

        return $this->response->title(trans('event::prize.names'))
            ->view('event::prize.index', true)
            ->data(compact('prizes', 'view'))
            ->output();
    }

    /**
     * Display prize.
     *
     * @param Request $request
     * @param Model   $prize
     *
     * @return Response
     */
    public function show(PrizeRequest $request, Prize $prize)
    {

        if ($prize->exists) {
            $view = 'event::prize.show';
        } else {
            $view = 'event::prize.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('event::prize.name'))
            ->data(compact('prize'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new prize.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PrizeRequest $request)
    {

        $prize = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('event::prize.name')) 
            ->view('event::prize.create', true) 
            ->data(compact('prize'))
            ->output();
    }

    /**
     * Create new prize.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PrizeRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $prize                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('event::prize.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('event/prize/' . $prize->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/event/prize'))
                ->redirect();
        }

    }

    /**
     * Show prize for editing.
     *
     * @param Request $request
     * @param Model   $prize
     *
     * @return Response
     */
    public function edit(PrizeRequest $request, Prize $prize)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('event::prize.name'))
            ->view('event::prize.edit', true)
            ->data(compact('prize'))
            ->output();
    }

    /**
     * Update the prize.
     *
     * @param Request $request
     * @param Model   $prize
     *
     * @return Response
     */
    public function update(PrizeRequest $request, Prize $prize)
    {
        try {
            $attributes = $request->all();

            $prize->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('event::prize.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('event/prize/' . $prize->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('event/prize/' . $prize->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the prize.
     *
     * @param Model   $prize
     *
     * @return Response
     */
    public function destroy(PrizeRequest $request, Prize $prize)
    {
        try {

            $prize->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('event::prize.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('event/prize/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('event/prize/' . $prize->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple prize.
     *
     * @param Model   $prize
     *
     * @return Response
     */
    public function delete(PrizeRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('event::prize.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('event/prize'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/event/prize'))
                ->redirect();
        }

    }

    /**
     * Restore deleted prizes.
     *
     * @param Model   $prize
     *
     * @return Response
     */
    public function restore(PrizeRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('event::prize.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/event/prize'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/event/prize/'))
                ->redirect();
        }

    }

}
